@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2000 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1999-2000, 2002 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  *
  * By using this file, you agree to the terms and conditions set
@@ -7,7 +7,7 @@
  * the sendmail distribution.
  *
  *
- *	$Id: mfapi.h,v 8.13.4.12 2000/09/09 02:11:48 ca Exp $
+ *	$Id: mfapi.h,v 8.13.4.16 2002/02/12 22:21:54 msk Exp $
  */
 
 /*
@@ -361,6 +361,17 @@ LIBMILTER_API int smfi_addrcpt __P((SMFICTX *, char *));
 
 LIBMILTER_API int smfi_delrcpt __P((SMFICTX *, char *));
 
+#if _FFR_SMFI_PROGRESS
+/*
+**  Send a "no-op" up to the MTA to tell it we're still alive, so long
+**  milter-side operations don't time out.
+**
+**	SMFICTX *ctx; Opaque context structure
+*/
+
+LIBMILTER_API int smfi_progress __P((SMFICTX *));
+#endif /* _FFR_SMFI_PROGRESS */
+
 /*
 **  Delete a recipient from the envelope
 **
