@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: comm.c,v 8.30.4.6 2000/10/05 22:44:01 gshapiro Exp $";
+static char id[] = "@(#)$Id: comm.c,v 8.30.4.7 2001/11/07 09:04:58 msk Exp $";
 #endif /* ! lint */
 
 #if _FFR_MILTER
@@ -73,7 +73,9 @@ mi_rd_cmd(sd, timeout, cmd, rlen, name)
 			*cmd = SMFIC_SELECT;
 			return NULL;
 		}
-		if ((len = MI_SOCK_READ(sd, data + i, sizeof data - i)) < 0)
+
+		if (MI_SOCK_READ_FAIL(len = MI_SOCK_READ(sd, data + i,
+		                                         sizeof data - i)))
 		{
 			smi_log(SMI_LOG_ERR,
 				"%s, mi_rd_cmd: read returned %d: %s",
@@ -133,7 +135,8 @@ mi_rd_cmd(sd, timeout, cmd, rlen, name)
 			free(buf);
 			return NULL;
 		}
-		if ((len = MI_SOCK_READ(sd, buf + i, expl - i)) < 0)
+		if (MI_SOCK_READ_FAIL(len = MI_SOCK_READ(sd, buf + i,
+		                                         expl - i)))
 		{
 			smi_log(SMI_LOG_ERR,
 				"%s: mi_rd_cmd: read returned %d: %s",
