@@ -1,5 +1,5 @@
 /*
-** Copyright (c) 1999-2000 Sendmail, Inc. and its suppliers.
+** Copyright (c) 1999-2000, 2003 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
 ** By using this file, you agree to the terms and conditions set
@@ -8,7 +8,7 @@
 */
 
 #ifndef lint
-static char id[] = "@(#)$Id: smndbm.c,v 8.40.4.3 2000/10/05 22:27:50 gshapiro Exp $";
+static char id[] = "@(#)$Id: smndbm.c,v 8.40.4.6 2003/03/20 23:25:15 ca Exp $";
 #endif /* ! lint */
 
 #include <fcntl.h>
@@ -506,6 +506,7 @@ smdb_ndbm_open(database, db_name, mode, mode_mask, sff, type, user_info,
 	SMDB_USER_INFO *user_info;
 	SMDB_DBPARAMS *db_params;
 {
+	bool lockcreated = FALSE;
 	int result;
 	int lock_fd;
 	SMDB_DATABASE *smdb_db;
@@ -530,15 +531,34 @@ smdb_ndbm_open(database, db_name, mode, mode_mask, sff, type, user_info,
 	if (result != SMDBE_OK)
 		return result;
 
+	if ((dir_stat_info.st_mode == ST_MODE_NOFILE ||
+	     pag_stat_info.st_mode == ST_MODE_NOFILE) &&
+	    bitset(mode, O_CREAT))
+		lockcreated = TRUE;
+
 	lock_fd = -1;
-# if O_EXLOCK
-	mode |= O_EXLOCK;
-# else /* O_EXLOCK */
 	result = smdb_lock_file(&lock_fd, db_name, mode, sff,
 				SMNDB_DIR_FILE_EXTENSION);
 	if (result != SMDBE_OK)
 		return result;
-# endif /* O_EXLOCK */
+
+	if (lockcreated)
+	{
+		int pag_fd;
+
+		/* Need to pre-open the .pag file as well with O_EXCL */
+		result = smdb_lock_file(&pag_fd, db_name, mode, sff,
+					SMNDB_PAG_FILE_EXTENSION);
+		if (result != SMDBE_OK)
+		{
+			(void) close(lock_fd);
+			return result;
+		}
+		(void) close(pag_fd);
+
+		mode |= O_TRUNC;
+		mode &= ~(O_CREAT|O_EXCL);
+	}
 
 	smdb_db = smdb_malloc_database();
 	if (smdb_db == NULL)
