@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1999, 2001 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  *
  * By using this file, you agree to the terms and conditions set
@@ -9,14 +9,14 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: debug.c,v 8.2 1999/07/26 04:04:09 gshapiro Exp $";
+static char id[] = "@(#)$Id: debug.c,v 8.2.16.1 2001/11/26 18:10:51 ca Exp $";
 #endif /* ! lint */
 
 #include <sendmail.h>
 
 u_char	tTdvect[100];	/* trace vector */
 
-#if _FFR_DPRINTF_
+#if _FFR_DPRINTF
 void
 /*VARARGS1*/
 #ifdef __STDC__
@@ -37,4 +37,4 @@ dflush()
 {
 	return fflush(stdout);
 }
-#endif /* _FFR_DPRINTF_ */
+#endif /* _FFR_DPRINTF */
