@@ -21,7 +21,7 @@ static char copyright[] =
 #endif /* ! lint */
 
 #ifndef lint
-static char id[] = "@(#)$Id: makemap.c,v 8.135.4.13 2000/10/05 23:00:50 gshapiro Exp $";
+static char id[] = "@(#)$Id: makemap.c,v 8.135.4.14 2001/10/03 22:11:41 ca Exp $";
 #endif /* ! lint */
 
 
@@ -53,7 +53,7 @@ BITMAP256 DontBlameSendmail;
 
 #define BUFSIZE		1024
 #if _FFR_DELIM
-# define ISSEP(c) ((sep == '\0' && isascii(c) && isspace(c)) || (c) == sep)
+# define ISSEP(c) (sep == '\0' ? isascii(c) && isspace(c) : (c) == sep)
 #else /* _FFR_DELIM */
 # define ISSEP(c) (isascii(c) && isspace(c))
 #endif /* _FFR_DELIM */
@@ -440,7 +440,7 @@ main(argc, argv)
 
 			if (*p != '\0')
 				*p++ = '\0';
-			while (ISSEP(*p))
+			while (*p != '\0' && ISSEP(*p))
 				p++;
 			if (!allowempty && *p == '\0')
 			{
