@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: bf_portable.c,v 8.25.4.6 2001/05/03 17:24:01 gshapiro Exp $";
+static char id[] = "@(#)$Id: bf_portable.c,v 8.25.4.7 2001/09/26 23:49:27 ca Exp $";
 #endif /* ! lint */
 
 #if SFIO
@@ -81,7 +81,7 @@ bfopen(filename, fmode, bsize, flags)
 	FILE *retval;
 	int fd, l;
 
-	fd = OPEN(filename, O_RDWR | O_CREAT | O_TRUNC, fmode, flags);
+	fd = OPEN(filename, O_RDWR|O_CREAT|O_TRUNC|QF_O_EXTRA, fmode, flags);
 	if (fd == -1)
 	{
 		/* errno is set implicitly by open */
