@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2001 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1998-2001, 2003 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -21,7 +21,7 @@ static char copyright[] =
 #endif /* ! lint */
 
 #ifndef lint
-static char id[] = "@(#)$Id: main.c,v 8.485.4.65 2001/07/20 00:53:00 gshapiro Exp $";
+static char id[] = "@(#)$Id: main.c,v 8.485.4.71 2003/03/20 23:12:26 ca Exp $";
 #endif /* ! lint */
 
 #define	_DEFINE
@@ -493,6 +493,8 @@ main(argc, argv, envp)
 	OldUmask = umask(022);
 	OpMode = MD_DELIVER;
 	FullName = getextenv("NAME");
+	if (FullName != NULL)
+		FullName = newstr(FullName);
 
 	/*
 	**  Initialize name server if it is going to be used.
@@ -1765,10 +1767,10 @@ main(argc, argv, envp)
 		dropenvelope(CurEnv, TRUE);
 
 #if DAEMON
-# if STARTTLS
+#  if STARTTLS
 		/* init TLS for server, ignore result for now */
 		(void) initsrvtls();
-# endif /* STARTTLS */
+#  endif /* STARTTLS */
 		p_flags = getrequests(CurEnv);
 
 		/* drop privileges */
@@ -1971,6 +1973,7 @@ main(argc, argv, envp)
 	{
 		int savederrors = Errors;
 		long savedflags = CurEnv->e_flags & EF_FATALERRS;
+		char buf[MAXLINE];
 
 		CurEnv->e_flags |= EF_GLOBALERRS;
 		CurEnv->e_flags &= ~EF_FATALERRS;
@@ -2014,6 +2017,11 @@ main(argc, argv, envp)
 			/* NOTREACHED */
 			return -1;
 		}
+
+		/* set message size */
+		snprintf(buf, sizeof buf, "%ld", CurEnv->e_msgsize);
+		define(macid("{msg_size}", NULL), newstr(buf), CurEnv);
+
 		Errors = savederrors;
 		CurEnv->e_flags |= savedflags;
 	}
@@ -3300,7 +3308,7 @@ testmodeline(line, e)
 				printf("Usage: /parse address\n");
 				return;
 			}
-			q = crackaddr(p);
+			q = crackaddr(p, e);
 			printf("Cracked address = ");
 			xputs(q);
 			printf("\nParsing %s %s address\n",
