@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2001 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1999-2003 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  *
  * By using this file, you agree to the terms and conditions set
@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: milter.c,v 8.50.4.53 2001/08/15 02:01:03 ca Exp $";
+static char id[] = "@(#)$Id: milter.c,v 8.50.4.70 2003/01/10 22:33:54 ca Exp $";
 #endif /* ! lint */
 
 #if _FFR_MILTER
@@ -26,6 +26,22 @@ static char id[] = "@(#)$Id: milter.c,v 8.50.4.53 2001/08/15 02:01:03 ca Exp $";
 #  define SM_FD_ISSET	FD_ISSET
 #  define SM_FD_SETSIZE	FD_SETSIZE
 
+#if _FFR_LOG_MILTER_DEBUG
+/*
+**  HACK for debugging milter problem on windows
+*/
+
+#undef dprintf
+
+void
+dprintf(fmt, ap)
+	const char *fmt;
+	va_list ap;
+{
+	sm_syslog(LOG_NOTICE, NOQID, fmt, ap);
+}
+#endif /* _FFR_LOG_MILTER_DEBUG */
+
 static void	milter_connect_timeout __P((void));
 static void	milter_error __P((struct milter *));
 static int	milter_open __P((struct milter *, bool, ENVELOPE *));
@@ -547,6 +563,7 @@ milter_open(m, parseonly, e)
 	}
 
 	/* protocol:filename or protocol:port@host */
+	memset(&addr, '\0', sizeof addr);
 	p = m->mf_conn;
 	colon = strchr(p, ':');
 	if (colon != NULL)
@@ -1491,7 +1508,7 @@ milter_reopen_df(e)
 		}
 
 		/* open writable */
-		if ((e->e_dfp = fopen(dfname, "w+")) == NULL)
+		if ((e->e_dfp = fopen(dfname, QF_WR_MODE)) == NULL)
 		{
 			MILTER_DF_ERROR("milter_reopen_df: fopen %s: %s");
 			return -1;
@@ -1545,7 +1562,7 @@ milter_reset_df(e)
 		MILTER_DF_ERROR("milter_reset_df: error closing %s: %s");
 		return -1;
 	}
-	else if ((e->e_dfp = fopen(dfname, "r")) == NULL)
+	else if ((e->e_dfp = fopen(dfname, QF_R_MODE)) == NULL)
 	{
 		MILTER_DF_ERROR("milter_reset_df: error reopening %s: %s");
 		return -1;
@@ -3298,7 +3315,7 @@ milter_data(e, state)
 						  "milter_data(%s): EOM ACK/NAK timeout\n",
 						  m->mf_name);
 				milter_error(m);
-				MILTER_CHECK_ERROR(continue);
+				MILTER_CHECK_ERROR(/* EMPTY */);
 				break;
 			}
 
