@@ -16,9 +16,9 @@
 
 #ifndef lint
 # if QUEUE
-static char id[] = "@(#)$Id: queue.c,v 8.343.4.62 2001/07/20 00:53:01 gshapiro Exp $ (with queueing)";
+static char id[] = "@(#)$Id: queue.c,v 8.343.4.65 2003/03/05 17:46:02 lijian Exp $ (with queueing)";
 # else /* QUEUE */
-static char id[] = "@(#)$Id: queue.c,v 8.343.4.62 2001/07/20 00:53:01 gshapiro Exp $ (without queueing)";
+static char id[] = "@(#)$Id: queue.c,v 8.343.4.65 2003/03/05 17:46:02 lijian Exp $ (without queueing)";
 # endif /* QUEUE */
 #endif /* ! lint */
 
@@ -202,7 +202,7 @@ queueup(e, announce)
 			else
 				(void) sleep(i % 32);
 		}
-		if (tfd < 0 || (tfp = fdopen(tfd, "w")) == NULL)
+		if (tfd < 0 || (tfp = fdopen(tfd, QF_W_MODE)) == NULL)
 		{
 			int save_errno = errno;
 
@@ -265,15 +265,15 @@ queueup(e, announce)
 
 			if (bitset(S_IWGRP, QueueFileMode))
 				oldumask = umask(002);
-			dfd = open(dfname, O_WRONLY|O_CREAT|O_TRUNC,
+			dfd = open(dfname, O_WRONLY|O_CREAT|O_TRUNC|QF_O_EXTRA,
 				   QueueFileMode);
 			if (bitset(S_IWGRP, QueueFileMode))
 				(void) umask(oldumask);
 		}
 #else /* _FFR_QUEUE_FILE_MODE */
-		dfd = open(dfname, O_WRONLY|O_CREAT|O_TRUNC, FileMode);
+		dfd = open(dfname, O_WRONLY|O_CREAT|O_TRUNC|QF_O_EXTRA, FileMode);
 #endif /* _FFR_QUEUE_FILE_MODE */
-		if (dfd < 0 || (dfp = fdopen(dfd, "w")) == NULL)
+		if (dfd < 0 || (dfp = fdopen(dfd, QF_W_MODE)) == NULL)
 			syserr("!queueup: cannot create data temp file %s, uid=%d",
 				dfname, geteuid());
 		if (fstat(dfd, &stbuf) < 0)
@@ -1214,7 +1214,7 @@ orderq(queuedir, doall)
 		}
 
 		/* open control file */
-		cf = fopen(qf, "r");
+		cf = fopen(qf, QF_R_MODE);
 
 		if (cf == NULL)
 		{
@@ -1905,7 +1905,7 @@ readqf(e)
 	*/
 
 	(void) strlcpy(qf, queuename(e, 'q'), sizeof qf);
-	qfp = fopen(qf, "r+");
+	qfp = fopen(qf, QF_RW_MODE);
 	if (qfp == NULL)
 	{
 		int save_errno = errno;
@@ -1933,6 +1933,7 @@ readqf(e)
 		return FALSE;
 	}
 
+
 	/*
 	**  Prevent locking race condition.
 	**
@@ -2363,7 +2364,7 @@ readqf(e)
 	*/
 
 	p = queuename(e, 'd');
-	e->e_dfp = fopen(p, "r");
+	e->e_dfp = fopen(p, QF_R_MODE);
 	if (e->e_dfp == NULL)
 	{
 		syserr("readqf: cannot open %s", p);
@@ -2374,6 +2375,8 @@ readqf(e)
 		if (fstat(fileno(e->e_dfp), &st) >= 0)
 		{
 			e->e_msgsize = st.st_size + hdrsize;
+			snprintf(buf, sizeof buf, "%ld", e->e_msgsize);
+			define(macid("{msg_size}", NULL), newstr(buf), e);
 			e->e_dfdev = st.st_dev;
 			e->e_dfino = st.st_ino;
 		}
@@ -2567,7 +2570,7 @@ print_single_queue(queuedir)
 
 		printf("%12s", w->w_name + 2);
 		(void) snprintf(qf, sizeof qf, "%s/%s", qd, w->w_name);
-		f = fopen(qf, "r");
+		f = fopen(qf, QF_R_MODE);
 		if (f == NULL)
 		{
 			printf(" (job completed)\n");
