@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2001 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1998-2001, 2003 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983, 1995-1997 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -15,9 +15,9 @@
 
 #ifndef lint
 # if SMTP
-static char id[] = "@(#)$Id: usersmtp.c,v 8.245.4.33 2001/05/23 18:53:09 ca Exp $ (with SMTP)";
+static char id[] = "@(#)$Id: usersmtp.c,v 8.245.4.36 2003/03/20 23:18:28 ca Exp $ (with SMTP)";
 # else /* SMTP */
-static char id[] = "@(#)$Id: usersmtp.c,v 8.245.4.33 2001/05/23 18:53:09 ca Exp $ (without SMTP)";
+static char id[] = "@(#)$Id: usersmtp.c,v 8.245.4.36 2003/03/20 23:18:28 ca Exp $ (without SMTP)";
 # endif /* SMTP */
 #endif /* ! lint */
 
@@ -121,6 +121,25 @@ smtpinit(m, mci, e, onlyhelo)
 		goto helo;
 
 	mci->mci_state = MCIS_OPENING;
+#if SASL
+	define(macid("{auth_type}", NULL), "", e);
+	define(macid("{auth_authen}", NULL), "", e);
+	define(macid("{auth_author}", NULL), "", e);
+	define(macid("{auth_ssf}", NULL), "", e);
+#endif /* SASL */
+#if STARTTLS
+	define(macid("{cert_issuer}", NULL), "", e);
+	define(macid("{cert_subject}", NULL), "", e);
+	define(macid("{cipher_bits}", NULL), "", e);
+	define(macid("{cipher}", NULL), "", e);
+	define(macid("{tls_version}", NULL), "", e);
+	define(macid("{verify}", NULL), "", e);
+# if _FFR_TLS_1
+	define(macid("{alg_bits}", NULL), "", e);
+	define(macid("{cn_issuer}", NULL), "", e);
+	define(macid("{cn_subject}", NULL), "", e);
+# endif /* _FFR_TLS_1 */
+#endif /* STARTTLS */
 
 	/*
 	**  Get the greeting message.
@@ -1873,6 +1892,9 @@ smtpdata(m, mci, e)
 			if (Verbose)
 				nmessage(">>> .");
 
+			sm_syslog(LOG_CRIT, e->e_id,
+				  "%.100s: SMTP DATA-1 protocol error: remote server returned response before final dot",
+				  CurHostName);
 			mci->mci_errno = EIO;
 			mci->mci_state = MCIS_ERROR;
 			mci_setstat(mci, EX_PROTOCOL, "5.5.0", NULL);
